/*
*****************************************************************************
**
**      Project     : Helmkamera
**
**      Component   : uC (ATmega32)
**
**      Modulename  : System
**
**      Filename    : init.c
**
**      Abstract    : Diese Datei beinhaltet die Initialisierungroutinen
**                    
**                    
**
**      Date        : 06.04.2005
**
**      Fertigstell :
**
**      Progs       : ICC_AVR 6.26B
**
**      Anderungen  :
**
**
**
**      CPU         :    
**                    interner Oszillator 16 MHz
**                    BOD enabled, Watchdog always on
**
*****************************************************************************
*/

#include "globals.h"
#include "stv.h"
#include "hoehe.h"
#include "spi.h"


void port_init(void)
{
 PORTA = 0x00;
 DDRA  = 0xFF;
 PORTB = 0x00;
 DDRB  = 0x01;
 PORTC = 0x18; //m103 output only
 DDRC  = 0xE7;
 PORTD = 0xF8;
 DDRD  = 0x00;
 PORTE = 0x80;
 DDRE  = 0x00;
 PORTF = 0x00;
 DDRF  = 0x00;
 PORTG = 0x00;
 DDRG  = 0x00;
}

//Watchdog initialisation
// prescale: 2048K cycles
void watchdog_init(void)
{
 WDR(); //this prevents a timout on enabling
 WDTCR = 0x1F; //schreiben vorbereiten ... nchste 4 takte muss geschrieben werden
 WDTCR = 0x0F; //WATCHDOG ENABLED - dont forget to issue WDRs
 WDR();
}

//TIMER0 initialisation - prescale:1024
// WGM: Normal
// desired value: 10mSec
// actual value:  9,984mSec (0,2%)
void timer0_init(void)
{
 TCCR0 = 0x00; //stop
 ASSR  = 0x00; //set async mode
 TCNT0 = 0x64; //set count
 OCR0  = 0x9C;
 TCCR0 = 0x07; //start timer
}


// ein Timer1 takt entspricht 4 us

//TIMER1 initialize - prescale:64
// WGM: 0) Normal, TOP=0xFFFF
// desired value: 1mSec
// actual value:  0,992mSec (0,8%)
void timer1_init(void)
{
 TCCR1B = 0x00; //stop
 TCNT1H = 0x00; //setup
 TCNT1L = 0x00;
 OCR1AH = 0x08;
 OCR1AL = 0x11;
 OCR1BH = 0x00;
 OCR1BL = 0x00;
 OCR1CH = 0x00;
 OCR1CL = 0x00;
 ICR1H  = 0x00;
 ICR1L  = 0x00;
 TCCR1A = 0x00;
 TCCR1B = 0x03; //start Timer
}

//TIMER2 initialisation - prescale:1
// WGM: PWM Phase correct
// desired value: 16000Hz
// actual value: 31372,549Hz (49,0%)
void timer2_init(void)
{
 TCCR2 = 0x00; //stop
 TCNT2 = 0x01; //setup
 OCR2  = 0x00;
 TCCR2 = 0x51; //start
}

//TIMER3 initialisation - prescale:256
// WGM: 0) Normal, TOP=0xFFFF
// luft durch als counter
// 
void timer3_init(void)
{
 TCCR3B = 0x00; //stop
 TCNT3H = 0xF9; //setup
 TCNT3L = 0x80;
 OCR3AH = 0xFF;
 OCR3AL = 0xFF;
 OCR3BH = 0xFF;
 OCR3BL = 0xFF;
 OCR3CH = 0xFF;
 OCR3CL = 0xFF;
 ICR3H  = 0xFF;
 ICR3L  = 0xFF;
 TCCR3A = 0x00;
 
 SETBIT(TCCR3B,ICNC3); //Input capture noise Canceler
 TCCR3B |= 0x04; //Clk/256 ... start timer
 
}



//UART0 initialisation
// desired baud rate: 19200
// actual: baud rate:19231 (0,2%)
// char size: 5 bits
// parity: Disabled
void uart0_init(void)
{
 UCSR0B = 0x00; //disable while setting baud rate
 UCSR0A = 0x00;
 UCSR0C = 0x06;
 UBRR0L = 0x33; //set baud rate lo
 UBRR0H = 0x00; //set baud rate hi
 UCSR0B = 0xD8;
}

//UART1 initialize
// desired baud rate:9600
// actual baud rate:9615 (0,2%)
// char size: 8 bit
// parity: Disabled
void uart1_init(void)
{
 UCSR1B = 0x00; //disable while setting baud rate
 UCSR1A = 0x00;
 UCSR1C = 0x06;
 UBRR1L = 0x67; //set baud rate lo
 UBRR1H = 0x00; //set baud rate hi
 UCSR1B = 0x98;
}


//------------------------------------------------------------------------------
void ADCinit(void)
//------------------------------------------------------------------------------
//ADC Wandler Starten BIT 6 von ADCSR startet Wandlung
{
//ADC initialize
 ADCSRA = 0x00; //disable adc
 ADMUX = 0x00; //select adc input 0
 ACSR  = 0x80;
 ADCSRA = 0xC7; //Dummy Conversion ... Start prescaler 128
 ADCSRA = 0xCF;
}


/*
**---------------------------------------------------------------------------
**
**  Abstract:
**     Initialisierungsroutine
**
**  Parameters:
**      None
**
**  Returns:
**      None
**
**---------------------------------------------------------------------------
*/
void init_devices(void)
{
 
 //stop errant interrupts until set up
 CLI(); //disable all interrupts
 XDIV  = 0x00; //xtal divider
 XMCRA = 0x00; //external memory
 EICRA = 0x00; //extended ext ints
 EICRB = 0x00; //extended ext ints
  EIMSK = 0x00;
 
 port_init();
 watchdog_init();
 timer0_init();
 timer1_init();
 timer2_init();
 timer3_init();
 uart0_init();
 uart1_init();
 ADCinit();
 
 init_stv();	//STV TEXTgenerator starten 
 
 SpiInit();
 Luftinit();

  
 MCUCR = 0x00;
 
 TIMSK = 0x01; //timer interrupt sources ... nur Timer 0 ovf interrupt
 
 //extended timer interrupt sources
 ETIFR = 0;
 ETIMSK = 0;
 SETBIT(TIMSK,TOIE1);
 SETBIT(ETIMSK,TICIE3);  //ICF Interrupt enable
 SETBIT(ETIMSK,TOV3);  //Timer 3 OVerflow interrupt enable
 
 SETBIT(EICRA,ISC21);   //EXT INT2 falling Edge
 
 SEI(); //re-enable interrupts
 //all peripherals are now initialised
}



